/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 21-Oct-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.arguments;

import java.util.HashMap;

import uk.ac.ic.doc.kevlar.publicInterfaces.INodeID;
import uk.ac.ic.doc.kevlar.publicUtil.collections.IImmutableMap;
import uk.ac.ic.doc.kevlar.publicUtil.collections.IImmutableSet;
import uk.ac.ic.doc.kevlar.publicUtil.collections.ImmutableHashMap;

/**
 * @author Daniel Burke
 *
 */
public class InputOuputNodeTuple implements IInputOutputNodeTuple 
{	
	private IImmutableMap<INodeID, IInputNode> inputsMap;
	private IImmutableMap<INodeID, IOutputNode> outputsMap;
	private IImmutableSet<IInputNode> inputsSet ;
	private IImmutableSet<IOutputNode> outputsSet ;
	
	public InputOuputNodeTuple(IImmutableSet<IInputNode> inputs, 
								IImmutableSet<IOutputNode> outputs)
	{
		inputsSet = inputs ;
		outputsSet = outputs ;
		
		HashMap<INodeID, IInputNode> tempInputs = new HashMap<INodeID, IInputNode>();
		HashMap<INodeID, IOutputNode> tempOutputs = new HashMap<INodeID, IOutputNode>();
		
		for( IInputNode node : inputs )
		{
			tempInputs.put( node.getNodeID(), node );						
		}
		
		for( IOutputNode node : outputs )
		{
			tempOutputs.put( node.getNodeID(), node );
		}
		
		inputsMap = new ImmutableHashMap<INodeID, IInputNode>(tempInputs);
		outputsMap = new ImmutableHashMap<INodeID, IOutputNode>(tempOutputs);
	}
	
	
	/**
	 * @return Returns the inputs.
	 */
	public IImmutableSet<IInputNode> getInputs() 
	{
		return inputsSet;
	}
	/**
	 * @return Returns the outputs.
	 */
	public IImmutableSet<IOutputNode> getOutputs() 
	{
		return outputsSet;		
	}
	
	public IInputNode getInputNode( INodeID id )
	{
		if(!inputsMap.containsKey(id))
		{
			throw new IllegalArgumentException("id is not an input node");
		}
		
		return inputsMap.get( id );
	}
	
	public IOutputNode getOutputNode( INodeID id )
	{
		if(!outputsMap.containsKey(id))
		{
			throw new IllegalArgumentException("id is not an output node");
		}
		
		return outputsMap.get( id );
	}
}
