/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 19-Oct-2004
 *
 */
package uk.ac.ic.doc.kevlar.api.programs.arguments.internal.argumentInstance;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentInstance;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IBasicArgumentValue;
import uk.ac.ic.doc.kevlar.api.programs.arguments.IBoolArgumentValue;
import uk.ac.ic.doc.kevlar.api.programs.arguments.ISetArgumentValue;
import uk.ac.ic.doc.kevlar.api.programs.arguments.ITextArgumentValue;
import uk.ac.ic.doc.kevlar.publicUtil.dom.XMLUtil;

/**
 * @author Daniel Burke
 *
 */
public final class ArgumentInstance implements IArgumentInstance, Serializable {
	/**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = -4256164576364644006L;


    private Map<String, IBasicArgumentValue> assignments;
	
	public ArgumentInstance(Map<String, IBasicArgumentValue> assignments)
	{
		if(assignments == null)
		{
			throw new IllegalArgumentException("param null");
		}
		this.assignments = 
			new HashMap<String, IBasicArgumentValue>(assignments);
	}
	
	public Map<String, IBasicArgumentValue> getAssigmentsMap()
	{
		return assignments;
	}
	
	public void putInstance(StringBuilder text, int depth)
	{
		depth = XMLUtil.putOpenElem(text, "arg-instance", depth);
		
		for(Map.Entry<String, IBasicArgumentValue> entry : assignments.entrySet())
		{
			depth = XMLUtil.putOpenElem(text, "pair", depth);

			XMLUtil.putElement(text, "name", entry.getKey(), depth);
			XMLUtil.putElement(text, "value", entry.getValue().toString(), depth);
			
			depth = XMLUtil.putCloseElem(text, "pair", depth);
		}
		
		depth = XMLUtil.putCloseElem(text, "arg-instance", depth);
	}
	

	public boolean has(String argName)
	{
	    return assignments.containsKey(argName);
	}
	
	public IBasicArgumentValue get(String argName)
	{
	    return assignments.get(argName);
	}
	
	public Boolean getBool(String argName)
	{
	    IBasicArgumentValue value = assignments.get(argName);
	    if(value == null)
	    {
	        return null;
	    }
	    
	    if(!(value instanceof IBoolArgumentValue))
	    {
	        throw new IllegalArgumentException("the argName provided must have a boolean value set");
	    }
	    
	    return Boolean.valueOf(((IBoolArgumentValue)value).getValue());
	}
	
	public String getText(String argName)
	{
	    IBasicArgumentValue value = assignments.get(argName);
	    if(value == null)
	    {
	        return null;
	    }
	    
	    if(!(value instanceof ITextArgumentValue))
	    {
	        throw new IllegalArgumentException("the argName provided must have a boolean value set");
	    }
	    
	    return ((ITextArgumentValue)value).getValue();
	}
	
	public String getSetValue(String argName)
	{
	    IBasicArgumentValue value = assignments.get(argName);

	    if(value == null)
	    {
	        return null;
	    }
	    
	    if(!(value instanceof ISetArgumentValue))
	    {
	        throw new IllegalArgumentException("the argName provided must have a boolean value set");
	    }
	    
	    return ((ISetArgumentValue)value).getValue();
	}
}
