/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 21-Oct-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.framework;

import java.util.SortedSet;

import uk.ac.ic.doc.kevlar.api.programs.arguments.IArgumentModel;
import uk.ac.ic.doc.kevlar.api.programs.returnValues.IReturnModel;
import uk.ac.ic.doc.kevlar.core.framework.internal.IProgram;
import uk.ac.ic.doc.kevlar.publicInterfaces.IProgramDescription;

/**
 * Represents a program repository
 * 
 * @author toa02
 *  
 */
public interface IProgramRepository {

    /**
     * Rehashes the directories and returns a Set of all the valid
     * IProgramDescriptions.
     * 
     * @return A Set of all the IProgramDescriptions in the system.
     * This can change
     */
    SortedSet<IProgramIDDescription> getProgramDescriptions();

    /**
     * Gets the ProgramArguments 
     * @param id
     * @return the ArgumentModel for the 
     */
    IArgumentModel getProgramArgument( IProgramID id );
    
    /**
     * Gets the Program's description 
     * @param id
     * @return the description for the program
     */
    IProgramDescription getProgramDescription( IProgramID id );

    /**
     * Gets the Program's return model
     * @param id
     * @return the return model for the program
     */
    IReturnModel getReturnModel(IProgramID id);
    
    /**
     * 
     * @param id is the program id we want to get the IProgram of
     * @return IProgram mapped to the id
     */
    IProgram getProgram( IProgramID id );
    
    /**
     * 
     * rehashes the newly loaded programs. 
     */
    void rehashPrograms();
    
	

}
