/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 21-Oct-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.framework.internal.loaders;

import java.io.File;
import java.util.Set;

import uk.ac.ic.doc.kevlar.core.framework.internal.IProgram;

/**
 * @author toa02
 *  
 */
public interface IProgramLoader {

    /**
     * Adds the given directory to those with programs loaded from.
     * 
     * @param f
     *            A directory to start watching.
     * @throws IllegalArgumentException
     *             if the provided argument is unreadable, or not a directory.
     * @throws NullPointerException
     *             if the provided argument is null.
     */
    void addDirectory(File f);

    /**
     * Removes the given directory from those which programs are loaded from.
     * Types that where in that directory will be unloaded by the system.
     * 
     * @param f
     *            A directory to stop looking at.
     * @throws IllegalArgumentException
     *             if the argument is not a directory that is being watched.
     * @throws NullPointerException
     *             if the provided argument is null.
     */
    void removeDirectory(File f);

    /**
     * This does <b>NOT</b> return a live pointer.
     * @return the loaded programs. This should be the main place to hang onto.
     */
    Set<IProgram> getPrograms();
}
