/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 25-Oct-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.core.framework.internal.loaders.directorywatcher;

import java.io.File;

/**
 * @author toa02
 *  
 */
public interface IDirectoryWatcher extends Runnable {

    /**
     * Add a directory to be watched, using the provided notify filter to
     * determine when things should be notified of changes.
     * 
     * @param f
     *            The directory to watch
     * @param inf
     *            The INotifierFilter to use to to filter when notifies should
     *            happen, and to notify on stuff. It will receive calls for each
     *            jar currently known to be in that directory.
     */
    void addDirectory(File f, INotifierFilter inf);

    /**
     * Remove a directory from being watched by the provided INotifierFilter.
     * The INotifierFilter will receive methods calls about each jar that has
     * been "removed" because it is not watching it any more.
     * 
     * @param f
     *            The directory to watch
     * @param inf
     *            The INotifierFilter to use to to filter when notifies should
     *            happen, and to notify on stuff.
     */
    void removeDirectory(File f, INotifierFilter inf);
}
