/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 20-Oct-2004
 *
 *
 */
package uk.ac.ic.doc.kevlar.core.framework.typecheck;

import java.util.Set;

/**
 * @author Steve Zymler
 *
 * ITypeTreeNode will offer functionality on the type tree
 * nodes contained within the ITypeTree.
 * 
 */
public interface ITypeTreeNode 
{
	/**
	 * 
	 * @return The immediate ITypeTreeNode superclass of the current
	 * 			ITypeTreeNode. 
	 */
	ITypeTreeNode getSuperNode();
	
	/**
	 * 
	 * @param superNode sets the immediate superclass of the node
	 * 			in the ITypeTree.
	 */
	void setSuperNode(ITypeTreeNode superNode);
	
	/**
	 * 
	 * @return the immediate sub classes of the node in the ITypeTree.
	 */
	Set<ITypeTreeNode> getSubNodes();
	
	/**
	 * 
	 * @param subNode is the subclass you want to add to the node
	 * 			in the ITypeTree.
	 */
	void addSubNode(ITypeTreeNode subNode);
	
	/**
	 * 
	 * @return the IInterfaceClass of the type this node represents.
	 */
	IInterfaceClass getInterfaceClass();
	
	/**
	 * 
	 * @param typeClass is the type you want to set that this node
	 * 			contains.
	 */
	void setInterfaceClass( IInterfaceClass typeClass );
}
