/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 23-Nov-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.keyboard.actions.autocomplete.program;

import java.util.ArrayList;

import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Point;

import uk.ac.ic.doc.kevlar.core.framework.IProgramIDDescription;
import uk.ac.ic.doc.kevlar.gui.GUIManager;
import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.AutoCompleteBox;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.IAutoCompleteListener;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.Item;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.actions.IAction;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.ProgramAutoCompleteState;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.State;
import uk.ac.ic.doc.kevlar.gui.internal.layout.LayoutManager;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.PipeLineManager;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.Program;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.ProgramListBox;
import uk.ac.ic.doc.kevlar.gui.internal.popup.PopUpBox;
import uk.ac.ic.doc.kevlar.gui.internal.popup.PopUpManager;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.IDeselectListener;

/**
 * @author Marc
 **/
public class ShowProgramAutoCompleteAction implements IAction, IAutoCompleteListener, IDeselectListener {

    protected PopUpBox popup;
    
    public boolean perform(Item event, State state, KeyEvent e) {
        
        if (PipeLineManager.getSingleton().isEnabled()) {
        
            ProgramListBox listbox = new ProgramListBox();
            listbox.setData(new ArrayList(GUIManager.getHial().getProgramTreeRoot().getSecond()));
            
            AutoCompleteBox autocomplete = new AutoCompleteBox();
            autocomplete.setListBox(listbox);
            autocomplete.addText(String.valueOf(e.character));
            autocomplete.setListener(this);
            autocomplete.getTextBox().setDeselectListener(this);
            
            popup = new PopUpBox();
            popup.setSize(new Point(150, 200));
            popup.setName("Add a new program");
            popup.setContents(autocomplete);
            popup.popUpAt(PipeLineManager.getSingleton().localToAbsolute(LayoutManager.getLayout().newProgramPosition()));
            
            KeyboardManager.getSingleton().setState(new ProgramAutoCompleteState(popup, autocomplete));
            
        }
        
        return false;
        
    }

    public void entryCompleted(Object entry) {
        
        if (entry instanceof IProgramIDDescription && popup != null) {
        
            PopUpManager.getSingleton().hidePopUp(popup);
            Program p = PipeLineManager.getSingleton().addProgram((IProgramIDDescription)entry);
            popup = null;
            PipeLineManager.getSingleton().selectProgram(p);
//            KeyboardManager.getSingleton().setState(new ProgramSelectedState(p));
            
        }
        
    }

    public void entryCancelled() {

        if (popup != null) {
        
            PopUpManager.getSingleton().hidePopUp(popup);
            popup = null;
            
        }
        
    }

    /* (non-Javadoc)
     * @see uk.ac.ic.doc.kevlar.gui.internal.widgets.IDeselectListener#onDeselect(uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject)
     */
    public void onDeselect(ADrawableObject object) {
        
        PopUpManager.getSingleton().hidePopUp(popup);
        
    }

}
