/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 29-Oct-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets;

import org.eclipse.swt.graphics.Point;

import sun.security.krb5.internal.crypto.h;
import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars.HorizontalScrollBar;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.scrollbars.VerticalScrollBar;


/**
 * @author Marc
 **/
public class ScrollingWindow extends AMouseListenerObject implements IScrollingWindow {

    protected HorizontalScrollBar horizScrollbar;
    protected VerticalScrollBar vertScrollbar;
    protected StretchyBox contents;
    
    public ScrollingWindow() {
        
        super();
        initialise();
        
    }

    public void initialise() {
        
        contents = new StretchyBox(true, true);
        super.addChild(contents);
                        
        horizScrollbar = new HorizontalScrollBar(this);
        horizScrollbar.setGapLeft();
        super.addChild(horizScrollbar);
        
        vertScrollbar = new VerticalScrollBar(this);
        vertScrollbar.setGapBottom();
        super.addChild(vertScrollbar);
        
    }
    
	public StretchyBox getContents() {
	    
	    return contents;
	    
	}
    
    public void setHorizontalPosition(int position) {
        
        contents.setPosition(new Point(-position, contents.getPosition().y));
        redraw();
        applyRedraw();
        
        updateScrollbarVisibility();
    }
    
    public void setVerticalPosition(int position) {
        
        contents.setPosition(new Point(contents.getPosition().x, -position));
        redraw();
        applyRedraw();
        
        updateScrollbarVisibility();
    }
     
    public int getHorizontalPosition() {
        
        return -contents.getPosition().x;
        
    }
    
    public int getVerticalPosition() {
        
        return -contents.getPosition().y;
        
    }
        
    public void setPadding(Point padding) {
        
        contents.setPadding(padding);
       
        
    }
    
    protected void updateScrollbarVisibility()
    {
    	if(horizScrollbar != null)
    	{
	    	if(horizScrollbar.isUsingFullSpace())
	    	{
	    		if(horizScrollbar.isVisible())
	    		{
		    		horizScrollbar.hide();
		    		if(vertScrollbar != null)
		    		{
		    			vertScrollbar.unsetGap();
		    		}
	    		}
	    	}
	    	else
	    	{
	    		if(!horizScrollbar.isVisible())
	    		{
		    		horizScrollbar.show();
		    		if(vertScrollbar != null)
		    		{
		    			vertScrollbar.setGapBottom();
		    		}
	    		}
	    	}
    	}
    	
	    if(vertScrollbar != null)
	    {
	       	if(vertScrollbar.isUsingFullSpace())
	    	{
	       		if(vertScrollbar.isVisible())
	       		{
		       		vertScrollbar.hide();
		       		if(horizScrollbar != null)
		       		{
		       			horizScrollbar.unsetGap();
		       		}
	       		}
	    	}
	    	else
	    	{
	       		if(!vertScrollbar.isVisible())
	       		{
		    		vertScrollbar.show();
		       		if(horizScrollbar != null)
		       		{
		       			horizScrollbar.setGapLeft();
		       		}
	       		}
	    	}
	    }
    }
    
    public void updateScrollBars() {

        horizScrollbar.updateFromWindow();
        vertScrollbar.updateFromWindow();
        
    }
    
    public void updateDimensions() {
        
        super.updateDimensions();
        horizScrollbar.setSize(area.width);
        vertScrollbar.setSize(area.height);
        horizScrollbar.updateFromWindow();
        vertScrollbar.updateFromWindow();
        
        updateScrollbarVisibility();
        
    }
    
    public void childDimensionsChanged(ADrawableObject child) {
        
        if (child == contents) {
            
            horizScrollbar.setMaximum(child.getSize().x);
            vertScrollbar.setMaximum(child.getSize().y);
            
            
        }
        
        updateScrollbarVisibility();
    }
    
    public void addChild(ADrawableObject child) {
        
        contents.addChild(child);
        
    }
    
    public void removeChild(ADrawableObject child) {
        
        contents.removeChild(child);
        
    }
    
    public void removeAllChildren() {
        
        contents.removeAllChildren();
        
    }
    
}
