/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 22-Dec-2004
 *
 */
package uk.ac.ic.doc.kevlar.gui.internal.widgets.textbox;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.caches.Utils;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.KeyboardManager;
import uk.ac.ic.doc.kevlar.gui.internal.keyboard.states.MultilineTextBoxState;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.IDeselectListener;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.ISelectListener;

/**
 * @author Marc
 **/
public class MultilineTextBox extends AMouseListenerObject implements ITextBox {

	private static final String EDIT_BOX_SKIN = "editbox";
	private static final String TRANSPARENT_EDIT_BOX_SKIN = "transparenteditbox";
    
    protected ScrollingTextArea textarea;
    protected boolean selected;
    protected ISelectListener selectListener;
    protected IDeselectListener deselectListener;
    
    public MultilineTextBox() {
        
        super();
        
        textarea = new ScrollingTextArea(this);
        textarea.setPosition(new Point(7, 2));
        textarea.setSizeFillBoth(new Point(2, 5));
        addChild(textarea);
        
        selected = false;
        selectListener = null;
        deselectListener = null;
        
    }
    
    public void setSelectListener(ISelectListener l) {
        
        selectListener = l;
        
    }
    
    public void setDeselectListener(IDeselectListener l) {
        
        deselectListener = l;
        
    }
    
    public void select() {
        
        selected = true;
        textarea.getTextArea().getCaret().permanentlyShow();
        redraw();
        
        if (selectListener != null) {
            
            selectListener.onSelect(this);
            
        }
        
    }
    
    public void deselect() {
        
        selected = false;
        textarea.getTextArea().getCaret().permanentlyHide();
        textarea.getTextArea().deselect();
        redraw();
        
        if (deselectListener != null) {
            
            deselectListener.onDeselect(this);
            
        }
        
    }
        
    public boolean isSelected() {
        
        return selected;
        
    }
    
    public TextArea getTextArea() {
        
        return textarea.getTextArea();
        
    }
    
    public void scrollTo(int y) {
        
        textarea.scrollTo(y);
        
    }
    
    public boolean capturesMouse() {
        
        return true;
        
    }
        
    public void mousePress(Point position, int button) {
        
        super.mousePress(position, button);
        
        if (!selected) {
        
            KeyboardManager.getSingleton().setState(new MultilineTextBoxState(this));
            
        }
                
    }
        
    public void drawBefore(GC gc, Rectangle redrawArea) {
        
        if (selected) {
            
            Utils.drawBox(gc, new Rectangle(0-redrawArea.x, 0-redrawArea.y, getSize().x, getSize().y), EDIT_BOX_SKIN);
            
        } else {
            
            Utils.drawBox(gc, new Rectangle(0-redrawArea.x, 0-redrawArea.y, getSize().x, getSize().y), TRANSPARENT_EDIT_BOX_SKIN);
            
        }        
        
    }

    public String getText() {
        
        return getTextArea().getText();
        
    }

    public void setText(String text) {
        
        getTextArea().setText(text);
        
    }
    
}
