/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * 
 * Created on 19-Nov-2004
 */
package uk.ac.ic.doc.kevlar.gui.internal.workbench;

import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import uk.ac.ic.doc.kevlar.gui.internal.ADrawableObject;
import uk.ac.ic.doc.kevlar.gui.internal.AMouseListenerObject;
import uk.ac.ic.doc.kevlar.gui.internal.pipeline.PipeLineManager;
import uk.ac.ic.doc.kevlar.gui.internal.showpane.ShowPaneManager;
import uk.ac.ic.doc.kevlar.gui.internal.taskpane.TaskPaneManager;
import uk.ac.ic.doc.kevlar.gui.internal.toolbar.ToolBarManager;
import uk.ac.ic.doc.kevlar.gui.internal.widgets.IDimensionListener;

/**
 * @author Daniel Burke
 *
 */
public class WorkBenchManager extends AMouseListenerObject implements IDimensionListener{
	
	private PipeLineManager plm;
	private TaskPaneManager tpm;
	private ShowPaneManager spm;
	private static WorkBenchManager singleton;
	
	
	public WorkBenchManager(PipeLineManager plm, TaskPaneManager tpm, ShowPaneManager spm)
	{
		this.plm = plm;
		this.tpm = tpm;
		this.spm = spm;
		
//		spm.addDimensionListener(this);
		
		setPosition(new Point(0, ToolBarManager.TOOL_BAR_HEIGHT));
		setAlignment(false, false);
		setSizeFillBoth(new Point(0, spm.getSize().y));
		
		addChild(plm);
		addChild(tpm);
	}
	
	public static void createSingleton(
			PipeLineManager plm, 
			TaskPaneManager tpm,
			ShowPaneManager spm)
	{
			
		
		if (singleton == null) {
			singleton = new WorkBenchManager(plm, tpm, spm);
		}
		
	}
	
    public static WorkBenchManager getSingleton() {
		
		return singleton;
		
	}
    
    public void updateSize()
	{
    	
	}

	public void objectDimensionsChanged(ADrawableObject object, Rectangle dimensions)
	{
//		setSizeFillBoth(new Point(0, object.getSize().y));
	} 
    
	public void showPaneResized(int height)
	{
		setSizeFillBoth(new Point(0, height));
	}
	
	public void updateDimensions()
    {
        super.updateDimensions();
        spm.checkSize();
    }
    
}
