/*******************************************************************************
* Copyright (c)                                                                *
*           2005   Tristan Allwood,                                            *
*           2005   Daniel Burke,                                               *
*           2005   Marc Hull,                                                  *
*           2005   Ekaterina Itskova,                                          *
*           2005   Steve Zymler                                                *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the                *
* "Software"), to deal in the Software without restriction, including          *
* without limitation the rights to use, copy, modify, merge, publish,          *
* distribute, sublicense, and/or sell copies of the Software, and to           *
* permit persons to whom the Software is furnished to do so, subject to        *
* the following conditions:                                                    *
*                                                                              *
* The above copyright notice and this permission notice shall be included      *
* in all copies or substantial portions of the Software.                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF                   *
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.       *
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY         *
* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,         *
* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE            *
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                       *
*******************************************************************************/
/*
 * Created on 29-Oct-2004 by toa02
 *
 */
package uk.ac.ic.doc.kevlar.publicInterfaces;

/**
 * Represents things you can do to the end of a pipe.
 * 
 * @author toa02
 * 
 */
public interface IPipeEnd {

    /**
     * Close the pipe. On return you are guaranteed nothing more can travel down
     * this pipe. If the pipe has been closed this method simply returns.
     */
    void close();

    /**
     * @return true if it is known that this pipe has been closed. This method
     *         will return immediately. For input pipes, this returns true if
     *         the pipe is closed, and there is nothing queued to be read from
     *         it.
     */
    boolean closed();

    /**
     * Add a close event listener to this pipe.
     * Be warned, the events coming back are in the PipeManager thread
     * and hold locks. You are <b>Strongly advised</b> not to make any calls
     * here that could take time to run, or require locks or call methods in
     * other IInput or IOutput pipes.
     * @param icel
     *            A Close Event Listener.
     */
    void addCloseEventListener(ICloseEventListener icel);

}
